<?php

namespace App\Models;

use App\Models\Enums\FileObjectType;
use App\Models\Scopes\CompanyScope;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOneThrough;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Schema;

class Folder extends Model
{
    use HasFactory, SoftDeletes;

    protected $guarded = ['id'];
    private mixed $children;
    public static function getColumns(): array
    {
        return Schema::getColumnListing((new self)->getTable());
    }

    protected static function booted(): void
    {
        static::addGlobalScope(new CompanyScope);
    }

    public function files()
    {
        return $this->hasMany(File::class)->orderByDesc("version");
    }

    public function namingRule(): BelongsTo
    {
        return $this->belongsTo(NamingRule::class);
    }

    public function parent(): BelongsTo
    {
        return $this->belongsTo(Folder::class, 'parent_id');
    }

    public function children(): HasMany
    {
        return $this->hasMany(Folder::class, 'parent_id');
    }

    public function container(): BelongsTo
    {
        return $this->belongsTo(Container::class, 'object_id');
    }

    public function library(): HasOneThrough
    {
        return $this->hasOneThrough(Library::class, Container::class, 'id', 'id', 'object_id', 'library_id');
    }
}
